/* 
        Dallas Semiconductor Button Application Program Interface

                       Unix Ver 0.01 (Beta) 2/22/94

               COPYRIGHT (C) 1994 DALLAS SEMICONDUCTOR COPORATION
*/

#include "dbapi.h"

#define TRUE  1
#define FALSE 0 

#define MAX_BUTTON_NUM     128        /* Maximum number of buttons reported */
#define ROM_STR_LEN        17                  /* Length of ROM data string */

uchar SearchPort(uchar, uchar);             /* Internal function prototypes */
uchar FindROMPort(RLSType *WS, uchar *ReqROM);
void  ReadConfig(void);                       
void  ConvertRomToString(uchar *RomPtr, char *RomStr);
void  AddLock(char *RomString);
void  RemoveLock(uchar LockLocation);
uchar SearchLockedList(uchar *RomString, uchar *LockLocation);
void  CheckForLock(uchar *RomPtr);

uchar      RomDataBuffer[MAX_BUTTON_NUM << 3];         /* Holds census data */
ROMLocType *ROMStr[MAX_BUTTON_NUM];     /* Keep track of which button where */
uchar      LockNum = 0;        /* Global representing number of locked ROMs */
dword      LockedList[MAX_BUTTON_NUM];                 /* Locked list array */
uchar      MaxPortNum[LastAdapter] = { MAX_SERIAL_PORTS, MAX_PARALLEL_PORTS };
uchar      *GRomPtr;                             /* Global ROM data pointer */
uchar      GNum = 0;                 /* Holds total number of buttons found */
uchar      ConfigSearched = FALSE;        /* Look at config file first time */
uchar      sm[LastAdapter][MAX_SERIAL_PORTS];            /* Ports to search */
char       *SPName[MAX_SERIAL_PORTS];                   /* TTY device names */
uchar      SPNum = 0;            /* Actual number of serial ports to search */
int        gsem_id;                         /* Global semaphore id variable */
                                            
uchar *take_census(uchar *num_pres)
{
   AdaptType IOType;                        /* Used to search adapter types */
   uchar     PortNum;
   uchar     *LRomPtr;
   uchar     x, i;
 
   if (!ConfigSearched)                        /* See which ports to search */
   {
      ReadConfig();
      ConfigSearched = TRUE;               /* Only want to search file once */
   }

   if (GNum)                 /* Free any memory associated with last census */
      for (i = 0; i < GNum; i++)
         free(ROMStr[i]);

   *num_pres = GNum = 0;                         /* Assume no buttons found */
   GRomPtr = RomDataBuffer;                /* Initialize global ROM pointer */

   for (IOType = 0; IOType < LastAdapter; IOType++)
   {
      for (PortNum = 0; PortNum < MaxPortNum[IOType]; PortNum++)
      {
         LRomPtr = GRomPtr;
         x = SearchPort((uchar) IOType, PortNum);/*Find all buttons on port */
         *num_pres += x;                       /* Update total number found */

         if (x)                   /* Did we find any buttons on this port ? */
         {
            for (i = 0; i < x; i++)      /* Indicate where these were found */
            {
               ROMStr[GNum + i] = (ROMLocType *) malloc(sizeof(ROMLocType));
               ROMStr[GNum + i]->PortType = IOType;
               ROMStr[GNum + i]->PortNumber = PortNum;
            
               memcpy(ROMStr[GNum + i]->RomData, LRomPtr, 8);   /* Copy ROM */

               LRomPtr += 8;              /* Point to next buttons ROM data */
            }
            GNum += x;           /* Add buttons found on this port to total */
         }
      }
   }
   return RomDataBuffer;                         /* Return pointer to ROM's */
}

dword GetKey(uchar *RomPtr)
{
   uchar i;
   dword GetKeyRes;

   GetKeyRes = 0;                                   /* Initialize key value */

   for (i = 0; i < 4; i++)                               /* Build key value */
      GetKeyRes += (dword) ((*(RomPtr + i) << (i << 3)));

   return GetKeyRes;
}

uchar SearchLockedList(uchar *RomPtr, uchar *LockLoc)
{
   uchar LockFound, i; 
   dword LockKey;

   if (!LockNum)          /* If no parts are locked, don't bother searching */
      return FALSE; 

   LockKey = GetKey(RomPtr);                                /* Get lock key */

   i = 0;                           /* Start searching at beginning of list */
   do 
   {
      LockFound = (LockKey == LockedList[i]);
   }
   while (!LockFound && (i++ < LockNum));  /* Search until found or no more */

   *LockLoc = i - 1;                               /* Set location variable */

   return LockFound;                             /* Return result of search */
}

TimeOutLock()
{
   semctl(gsem_id, 0, SETVAL, 1);                   /* Set semval back to 1 */
}

void CheckForLock(uchar *RomPtr)
{
   int           tsem_id, tsemflg;
   struct sembuf tsops;
   uchar         LockLoc;

   if (SearchLockedList(RomPtr, &LockLoc))           /* See if we have lock */
      return; 

   tsemflg = IPC_EXCL | OUR_SEM_PERMS;
   tsem_id = semget(GetKey(RomPtr), 1, tsemflg);
   
   if (tsem_id == -1)                           /* ROM not locked by anyone */ 
      return;
 
   tsops.sem_num = 0;                            /* 0'th semaphore in array */
   tsops.sem_op = -1;                               /* Decrement value by 1 */
   tsops.sem_flg = SEM_UNDO;             /* Undo in case of bad termination */

   signal(SIGALRM, TimeOutLock);                   /* Time out runaway lock */
   alarm(5);

   gsem_id = tsem_id;                           /* Used by time out routine */
   semop(tsem_id, &tsops, 1);                           /* Decrement semval */

   alarm(0);
   signal(SIGALRM, SIG_IGN);                         /* Cancel alarm signal */

   semctl(tsem_id, 0, SETVAL, 1);                   /* Set semval back to 1 */
}

uchar lock_rom(uchar *rom_ptr, dword time_out)
{
   uchar         LockRes, LockLoc;
   RLSType       RLStruct;
   char          RomStr[ROM_STR_LEN];         /* Hold string for conversion */
   int           tsem_id, tsemflg;
   dword         LockKey;
   struct sembuf tsops;

   LockRes = FALSE;                                       /* Assume failure */

   if (FindROMPort(&RLStruct, rom_ptr) && (LockNum < MAX_BUTTON_NUM))
   {
      if (!SearchLockedList(rom_ptr,&LockLoc))/*See if we already have lock */
      {
         LockKey = GetKey(rom_ptr);

         if (!time_out)            /* Make sure we wait some amount of time */
            time_out = 5000;

         tsemflg = IPC_CREAT | IPC_EXCL | OUR_SEM_PERMS;
         tsem_id = semget(LockKey, 1, tsemflg);
   
         if (tsem_id == -1)                            /* May already exist */
         {
            tsemflg = IPC_EXCL | OUR_SEM_PERMS;
            tsem_id = semget(LockKey, 1, tsemflg);
         }
         else                /* If just created initialize semaphore values */
            semctl(tsem_id, 0, SETVAL, 1);             /* Initialize semval */
   
         tsops.sem_num = 0;                      /* 0'th semaphore in array */
         tsops.sem_op = -1;                         /* Decrement value by 1 */
         tsops.sem_flg = SEM_UNDO;       /* Undo in case of bad termination */
      
         signal(SIGALRM, TimeOutLock);             /* Time out runaway lock */
         alarm(time_out / 1000 + 1);
      
         gsem_id = tsem_id;                     /* Used by time out routine */
         semop(tsem_id, &tsops, 1);                     /* Decrement semval */
      
         alarm(0);
         signal(SIGALRM, SIG_IGN);                   /* Cancel alarm signal */
      
         LockedList[LockNum++] = LockKey;               /* Add to lock list */ 

         LockRes = TRUE;                                   /* ROM is locked */
      }
      else
         LockRes = TRUE;                          /* ROM was already locked */
   }
   return LockRes;                 /* Return result of ROM locking function */
}

uchar unlock_rom(uchar *rom_ptr)
{
   uchar LockLoc, i;
   int   tsem_id, tsemflg;

   if (SearchLockedList(rom_ptr, &LockLoc))          /* See if we have lock */
   {
      tsemflg = IPC_EXCL | OUR_SEM_PERMS;
      tsem_id = semget(GetKey(rom_ptr), 1, tsemflg);

      semctl(tsem_id, 0, SETVAL, 1);                       /* Free the lock */

      for (i = LockLoc; i < LockNum; i++)          /* Remove lock from list */
         LockedList[i] = LockedList[i + 1];

      --LockNum;                           /* Indicate 1 less button locked */
   }

   return TRUE;
}

uchar read_1425_sp(uchar *scr_dta, uchar *rom_ptr)
{
   uchar        i;
   uchar        *LRomPtr;
   uchar        FRes;
   RLSType      RLStruct;
   DS1412PACKET ScratchPacket;

   CheckForLock(rom_ptr);   /* If ROM is locked, wait for it to be unlocked */

   FRes = FALSE;                                          /* Assume failure */
   if (FindROMPort(&RLStruct, rom_ptr))            /* See if part was found */
   {
      switch (RLStruct.PortType)             /* What adapter was it in last */
      {
         case SerialAdapter:                   /* Wer'e talking to a DS1412 */
            ScratchPacket.PacketLength = 76;      /* Length of transmission */
            ScratchPacket.RomCommand = 0x55;          /* ROM access command */

            memcpy(ScratchPacket.RomData, rom_ptr, 8);          /* ROM data */

            ScratchPacket.CommandLength = 3;     /* Number of command bytes */ 

            ScratchPacket.CommandData[0] = 0x69; /* Read scratchpad command */
            ScratchPacket.CommandData[1] = 0xC0;        /* Starting address */ 
            ScratchPacket.CommandData[2] = 0x3F;       /* Confirmation byte */ 
 
            ScratchPacket.DataPacketLength = 64;      /* Data packet length */
            memset(ScratchPacket.DataPacket, 0xFF, 64);       /* Data bytes */

            FRes = XmitDS1412Data(&ScratchPacket, RLStruct.PortNumber); 
           
            if (FRes)             /* Copy data to user buffer if successful */
               memcpy(scr_dta, ScratchPacket.DataPacket, 64);
         break;

         case ParallelAdapter:     /* Wer'e talking to a DS1410X or DS1410D */
            dskeyopen();          /* Get control of access system functions */
            dssetup(RLStruct.PortNumber);              /* Select port value */

            LRomPtr = dsromdata();        /* Get pointer to ROM data buffer */
            memcpy(LRomPtr, rom_ptr, 8);        /* Copy in callers ROM data */

            if (dsaccess())                /* Select specific device on bus */
            {
               dsdatabyte(0x69);       /* Send read scratchpad command byte */
               dsdatabyte(0xC0);             /* Send starting address value */
               dsdatabyte(0x3F);   /* Send ones complement for verification */

               for (i = 0; i < 64; i++)             /* Read scratchpad data */
                  *(scr_dta + i) = dsdatabyte(0xFF);   

               FRes = TRUE;                        /* Set success condition */
            }
            dskeyclose();     /* Release control of access system functions */
         break;

         default:
         break;
      }
   }
   return FRes;                         /* Return result of scratchpad read */
}

uchar write_1425_sp(uchar *scr_dta, uchar *rom_ptr)
{
   uchar        i;
   uchar        *LRomPtr;
   uchar        FRes;
   RLSType      RLStruct;
   DS1412PACKET ScratchPacket;

   CheckForLock(rom_ptr);   /* If ROM is locked, wait for it to be unlocked */

   FRes = FALSE;                                          /* Assume failure */
   if (FindROMPort(&RLStruct, rom_ptr))            /* See if part was found */
   {
      switch (RLStruct.PortType)             /* What adapter was it in last */
      {
         case SerialAdapter:                   /* Wer'e talking to a DS1412 */
            ScratchPacket.PacketLength = 76;      /* Length of transmission */
            ScratchPacket.RomCommand = 0x55;          /* ROM access command */

            memcpy(ScratchPacket.RomData, rom_ptr, 8);          /* ROM data */

            ScratchPacket.CommandLength = 3;     /* Number of command bytes */ 

            ScratchPacket.CommandData[0] = 0x96; /*Write scratchpad command */
            ScratchPacket.CommandData[1] = 0xC0;        /* Starting address */ 
            ScratchPacket.CommandData[2] = 0x3F;       /* Confirmation byte */ 
 
            ScratchPacket.DataPacketLength = 64;      /* Data packet length */
            memcpy(ScratchPacket.DataPacket, scr_dta, 64);    /* Data bytes */

            FRes = XmitDS1412Data(&ScratchPacket, RLStruct.PortNumber); 
         break;

         case ParallelAdapter:     /* Wer'e talking to a DS1410X or DS1410D */
            dskeyopen();          /* Get control of access system functions */
            dssetup(RLStruct.PortNumber);              /* Select port value */

            LRomPtr = dsromdata();        /* Get pointer to ROM data buffer */
            memcpy(LRomPtr, rom_ptr, 8);        /* Copy in callers ROM data */

            if (dsaccess())                /* Select specific device on bus */
            {
               dsdatabyte(0x96);      /* Send write scratchpad command byte */
               dsdatabyte(0xC0);             /* Send starting address value */
               dsdatabyte(0x3F);   /* Send ones complement for verification */

               for (i = 0; i < 64; i++)             /* Send scratchpad data */
                  dsdatabyte(*(scr_dta + i));

               FRes = TRUE;                        /* Set success condition */
            }
            dskeyclose();     /* Release control of access system functions */
         break;

         default:
         break;
      }
   }
   return FRes;                        /* Return result of scratchpad write */
}

uchar read_subkey(uchar key_num, uchar *key_dta, uchar *rom_ptr)
{
   uchar        i;
   uchar        *LRomPtr;
   uchar        FRes;
   RLSType      RLStruct;
   DS1412PACKET SubkeyPacket;

   CheckForLock(rom_ptr);   /* If ROM is locked, wait for it to be unlocked */

   FRes = FALSE;                                          /* Assume failure */
   if (FindROMPort(&RLStruct, rom_ptr))            /* See if part was found */
   {
      switch (RLStruct.PortType)             /* What adapter was it in last */
      {
         case SerialAdapter:                   /* Wer'e talking to a DS1412 */
            SubkeyPacket.PacketLength = 76;       /* Length of transmission */
            SubkeyPacket.RomCommand = 0x55;           /* ROM access command */

            memcpy(SubkeyPacket.RomData, rom_ptr, 8);           /* ROM data */

            SubkeyPacket.CommandLength = 3;      /* Number of command bytes */

            SubkeyPacket.CommandData[0] = 0x66;      /* Read subkey command */
            SubkeyPacket.CommandData[1] = (key_num << 6) | 0x10;
            SubkeyPacket.CommandData[2] = ~SubkeyPacket.CommandData[1];
 
            SubkeyPacket.DataPacketLength = 64;     /* Number of data bytes */

            memset(SubkeyPacket.DataPacket, 0xFF, 8);           /* ID bytes */
            memcpy(SubkeyPacket.DataPacket+8, key_dta+8, 8);    /* Password */
            memset(SubkeyPacket.DataPacket+16, 0xFF, 48);    /* Secure data */

            FRes = XmitDS1412Data(&SubkeyPacket, RLStruct.PortNumber); 
           
            if (FRes)    /* Copy data to user supplied buffer if successful */
            {
               memcpy(key_dta, SubkeyPacket.DataPacket, 8);
               memcpy(key_dta+16, SubkeyPacket.DataPacket+16, 48);
            }
         break;

         case ParallelAdapter:     /* Wer'e talking to a DS1410X or DS1410D */
            dskeyopen();          /* Get control of access system functions */
            dssetup(RLStruct.PortNumber);              /* Select port value */

            LRomPtr = dsromdata();        /* Get pointer to ROM data buffer */
            memcpy(LRomPtr, rom_ptr, 8);        /* Copy in callers ROM data */

            if (dsaccess())                /* Select specific device on bus */
            {
               dsdatabyte(0x66);           /* Send read subkey command byte */
               dsdatabyte((uchar) ((key_num << 6) | 0x10)); /* Address byte */
               dsdatabyte((uchar) ~((key_num << 6) | 0x10));   /* Ones comp */

               for (i = 0; i < 8; i++)                     /* Read ID field */
                  *(key_dta + i) = dsdatabyte(0xFF);      

               for (i = 8; i < 16; i++)              /* Send password field */
                  dsdatabyte(*(key_dta + i));

               for (i = 16; i < 64; i++)          /* Read secure data field */
                  *(key_dta + i) = dsdatabyte(0xFF);      

               FRes = TRUE;                        /* Set success condition */
            }
            dskeyclose();     /* Release control of access system functions */
         break;

         default:
         break;
      }
   }
   return FRes;                             /* Return result of subkey read */
}

uchar write_subkey(uchar key_num, uchar *key_dta, uchar *rom_ptr)
{
   uchar        i;
   uchar        *LRomPtr;
   uchar        FRes;
   RLSType      RLStruct;
   DS1412PACKET SubkeyPacket;

   CheckForLock(rom_ptr);   /* If ROM is locked, wait for it to be unlocked */

   FRes = FALSE;                                          /* Assume failure */
   if (FindROMPort(&RLStruct, rom_ptr))            /* See if part was found */
   {
      switch (RLStruct.PortType)             /* What adapter was it in last */
      {
         case SerialAdapter:                   /* Wer'e talking to a DS1412 */
            SubkeyPacket.PacketLength = 76;       /* Length of transmission */
            SubkeyPacket.RomCommand = 0x55;           /* ROM access command */

            memcpy(SubkeyPacket.RomData, rom_ptr, 8);           /* ROM data */

            SubkeyPacket.CommandLength = 3;      /* Number of command bytes */

            SubkeyPacket.CommandData[0] = 0x99;     /* Write subkey command */
            SubkeyPacket.CommandData[1] = (key_num << 6) | 0x10;
            SubkeyPacket.CommandData[2] = ~SubkeyPacket.CommandData[1];
 
            SubkeyPacket.DataPacketLength = 64;     /* Number of data bytes */

            memcpy(SubkeyPacket.DataPacket, key_dta, 8);        /* ID bytes */
            memcpy(SubkeyPacket.DataPacket+8, key_dta+8, 8);    /* Password */
            memcpy(SubkeyPacket.DataPacket+16, key_dta+16, 48); /* Sec data */

            FRes = XmitDS1412Data(&SubkeyPacket, RLStruct.PortNumber); 
         break;

         case ParallelAdapter:     /* Wer'e talking to a DS1410X or DS1410D */
            dskeyopen();          /* Get control of access system functions */
            dssetup(RLStruct.PortNumber);              /* Select port value */

            LRomPtr = dsromdata();        /* Get pointer to ROM data buffer */
            memcpy(LRomPtr, rom_ptr, 8);        /* Copy in callers ROM data */

            if (dsaccess())                /* Select specific device on bus */
            {
               dsdatabyte(0x99);          /* Send write subkey command byte */
               dsdatabyte((uchar) ((key_num << 6) | 0x10)); /* Address byte */
               dsdatabyte((uchar) (~((key_num << 6) | 0x10))); /* Ones comp */

               for (i = 0; i < 8; i++)                     /* Read ID field */
                  *(key_dta + i) = dsdatabyte(0xFF);      

               for (i = 8; i < 16; i++)              /* Send password field */
                  dsdatabyte(*(key_dta + i));

               for (i = 16; i < 64; i++)      /* Send new secure data field */
                  dsdatabyte(*(key_dta + i));

               FRes = TRUE;                        /* Set success condition */
            }
            dskeyclose();     /* Release control of access system functions */
         break;

         default:
         break;
      }
   }
   return FRes;                             /* Return result of subkey read */
}

uchar reprogram_subkey(uchar key_num, uchar *key_dta, uchar *rom_ptr)
{
   uchar        i;
   uchar        *LRomPtr;
   uchar        FRes;
   RLSType      RLStruct;
   DS1412PACKET RepkeyPacket;

   CheckForLock(rom_ptr);   /* If ROM is locked, wait for it to be unlocked */

   FRes = FALSE;                                          /* Assume failure */
   if (FindROMPort(&RLStruct, rom_ptr))            /* See if part was found */
   {
      switch (RLStruct.PortType)             /* What adapter was it in last */
      {
         case SerialAdapter:                   /* Wer'e talking to a DS1412 */
            RepkeyPacket.PacketLength = 44;       /* Length of transmission */
            RepkeyPacket.RomCommand = 0x55;           /* ROM access command */

            memcpy(RepkeyPacket.RomData, rom_ptr, 8);           /* ROM data */

            RepkeyPacket.CommandLength = 3;      /* Number of command bytes */

            RepkeyPacket.CommandData[0] = 0x5A; /* Reprogram subkey command */
            RepkeyPacket.CommandData[1] = (key_num << 6);
            RepkeyPacket.CommandData[2] = ~RepkeyPacket.CommandData[1];
 
            RepkeyPacket.DataPacketLength = 32;     /* Bytes in data packet */

            memset(RepkeyPacket.DataPacket, 0xFF, 8); /* For reading old ID */
            memcpy(RepkeyPacket.DataPacket+8, key_dta, 8);  /* Old ID bytes */
            memcpy(RepkeyPacket.DataPacket+16, key_dta+8, 8);     /* New ID */
            memcpy(RepkeyPacket.DataPacket+24, key_dta+16, 8);  /* New Pass */

            FRes = XmitDS1412Data(&RepkeyPacket, RLStruct.PortNumber); 
         break;

         case ParallelAdapter:     /* Wer'e talking to a DS1410X or DS1410D */
            dskeyopen();          /* Get control of access system functions */
            dssetup(RLStruct.PortNumber);              /* Select port value */

            LRomPtr = dsromdata();        /* Get pointer to ROM data buffer */
            memcpy(LRomPtr, rom_ptr, 8);        /* Copy in callers ROM data */

            if (dsaccess())                /* Select specific device on bus */
            {
               dsdatabyte(0x5A);           /* Reprogram subkey command byte */
               dsdatabyte((uchar) (key_num << 6));        /* Specify subkey */
               dsdatabyte((uchar) ~(key_num << 6));      /* Ones complement */

               for (i = 0; i < 8; i++)                     /* Read ID field */
                  *(key_dta + i) = dsdatabyte(0xFF);      

               for (i = 0; i < 24; i++)              /* Send reprogram data */
                  dsdatabyte(*(key_dta + i));

               FRes = TRUE;                        /* Set success condition */
            }
            dskeyclose();     /* Release control of access system functions */
         break;

         default:
         break;
      }
   }
   return FRes;                             /* Return result of subkey read */
}

unsigned char mblks[9][8] =       /* move block 64-bit block selector codes */
{                               
   0x9a,0x9a,0xb3,0x9d,0x64,0x6e,0x69,0x4c,
   0x9a,0x9a,0x4C,0x62,0x9b,0x91,0x69,0x4c,
   0x9a,0x65,0xb3,0x62,0x9b,0x6e,0x96,0x4c,
   0x6a,0x6a,0x43,0x6d,0x6b,0x61,0x66,0x43,
   0x95,0x95,0xbc,0x92,0x94,0x9e,0x99,0xbC,
   0x65,0x9a,0x4c,0x9d,0x64,0x91,0x69,0xb3,
   0x65,0x65,0xb3,0x9d,0x64,0x6e,0x96,0xb3,
   0x65,0x65,0x4c,0x62,0x9b,0x91,0x96,0xb3,
   0x56,0x56,0x7f,0x51,0x57,0x5d,0x5a,0x7f
};

uchar move_block(uchar key_num, uchar *key_dta, uchar *rom_ptr, uchar blocknum)
{
   uchar        i;
   uchar        *LRomPtr;
   uchar        FRes;
   RLSType      RLStruct;
   DS1412PACKET MovBlkPacket;

   CheckForLock(rom_ptr);   /* If ROM is locked, wait for it to be unlocked */

   FRes = FALSE;                                          /* Assume failure */
   if (FindROMPort(&RLStruct, rom_ptr))            /* See if part was found */
   {
      switch (RLStruct.PortType)             /* What adapter was it in last */
      {
         case SerialAdapter:                   /* Wer'e talking to a DS1412 */
            MovBlkPacket.PacketLength = 28;       /* Length of transmission */
            MovBlkPacket.RomCommand = 0x55;           /* ROM access command */

            memcpy(MovBlkPacket.RomData, rom_ptr, 8);           /* ROM data */

            MovBlkPacket.CommandLength = 3;      /* Number of command bytes */

            MovBlkPacket.CommandData[0] = 0x3C;  /* Move block command byte */
            MovBlkPacket.CommandData[1] = (key_num << 6);
            MovBlkPacket.CommandData[2] = ~MovBlkPacket.CommandData[1];
 
            MovBlkPacket.DataPacketLength = 16;     /* Number of data bytes */

            memcpy(MovBlkPacket.DataPacket, mblks[blocknum], 8);    /* Code */
            memcpy(MovBlkPacket.DataPacket+8, key_dta+8, 8);    /* Password */

            FRes = XmitDS1412Data(&MovBlkPacket, RLStruct.PortNumber); 
         break;

         case ParallelAdapter:     /* Wer'e talking to a DS1410X or DS1410D */
            dskeyopen();          /* Get control of access system functions */
            dssetup(RLStruct.PortNumber);              /* Select port value */

            LRomPtr = dsromdata();        /* Get pointer to ROM data buffer */
            memcpy(LRomPtr, rom_ptr, 8);        /* Copy in callers ROM data */

            if (dsaccess())                /* Select specific device on bus */
            {
               dsdatabyte(0x3C);                 /* Move block command byte */
               dsdatabyte((uchar) (key_num << 6));        /* Specify subkey */
               dsdatabyte((uchar) ~(key_num << 6));      /* Ones complement */

               for (i = 0; i < 8; i++) 
                  dsdatabyte(mblks[blocknum][i]);

               for (i = 8; i < 16; i++)                    /* Send password */
                  dsdatabyte(*(key_dta + i));

               FRes = TRUE;                        /* Set success condition */
            }
            dskeyclose();     /* Release control of access system functions */
         break;

         default:
         break;
      }
   }
   return FRes;                             /* Return result of subkey read */
}

uchar read_1427_sp(uchar *scr_dta, uchar *rom_ptr)
{
   uchar        i;
   uchar        *LRomPtr;
   uchar        FRes;
   RLSType      RLStruct;
   DS1412PACKET ScratchPacket;

   CheckForLock(rom_ptr);   /* If ROM is locked, wait for it to be unlocked */

   FRes = FALSE;                                          /* Assume failure */
   if (FindROMPort(&RLStruct, rom_ptr))            /* See if part was found */
   {
      switch (RLStruct.PortType)             /* What adapter was it in last */
      {
         case SerialAdapter:                   /* Wer'e talking to a DS1412 */
            ScratchPacket.PacketLength = 45;      /* Length of transmission */
            ScratchPacket.RomCommand = 0x55;          /* ROM access command */

            memcpy(ScratchPacket.RomData, rom_ptr, 8);          /* ROM data */

            ScratchPacket.CommandLength = 1;         /* Command byte number */

            ScratchPacket.CommandData[0] = 0xAA; /* Read scratchpad command */
            
            ScratchPacket.DataPacketLength = 35;    /* Number of data bytes */
 
            memset(ScratchPacket.DataPacket, 0xFF, 35);       /* Data bytes */

            FRes = XmitDS1412Data(&ScratchPacket, RLStruct.PortNumber); 
           
            if (FRes)             /* Copy data to user buffer if successful */
               memcpy(scr_dta, ScratchPacket.DataPacket, 35);
         break;

         case ParallelAdapter:     /* Wer'e talking to a DS1410X or DS1410D */
            dskeyopen();          /* Get control of access system functions */
            dssetup(RLStruct.PortNumber);              /* Select port value */

            LRomPtr = dsromdata();        /* Get pointer to ROM data buffer */
            memcpy(LRomPtr, rom_ptr, 8);        /* Copy in callers ROM data */

            if (dsaccess())                /* Select specific device on bus */
            {
               dsdatabyte(0xAA);       /* Send read scratchpad command byte */

               for (i = 0; i < 35; i++)             /* Read scratchpad data */
                  *(scr_dta + i) = dsdatabyte(0xFF);   

               FRes = TRUE;                        /* Set success condition */
            }
            dskeyclose();     /* Release control of access system functions */
         break;

         default:
         break;
      }
   }
   return FRes;                         /* Return result of scratchpad read */
}

uchar write_1427_sp(uchar *scr_dta, uchar *rom_ptr, uchar page_num)
{
   uchar        i;
   uchar        *LRomPtr;
   uchar        FRes;
   RLSType      RLStruct;
   DS1412PACKET ScratchPacket;

   CheckForLock(rom_ptr);   /* If ROM is locked, wait for it to be unlocked */

   FRes = FALSE;                                          /* Assume failure */
   if (FindROMPort(&RLStruct, rom_ptr))            /* See if part was found */
   {
      switch (RLStruct.PortType)             /* What adapter was it in last */
      {
         case SerialAdapter:                   /* Wer'e talking to a DS1412 */
            ScratchPacket.PacketLength = 44;      /* Length of transmission */
            ScratchPacket.RomCommand = 0x55;          /* ROM access command */

            memcpy(ScratchPacket.RomData, rom_ptr, 8);          /* ROM data */

            ScratchPacket.CommandLength = 3;     /* Number of command bytes */

            ScratchPacket.CommandData[0] = 0x0F; /*write scratchpad command */
            ScratchPacket.CommandData[1] = page_num << 5;     /* Target low */
            ScratchPacket.CommandData[2] = page_num >> 3;    /* Target high */
 
            ScratchPacket.DataPacketLength = 32;
            memcpy(ScratchPacket.DataPacket, scr_dta, 32);    /* Data bytes */

            FRes = XmitDS1412Data(&ScratchPacket ,RLStruct.PortNumber); 
         break;

         case ParallelAdapter:     /* Wer'e talking to a DS1410X or DS1410D */
            dskeyopen();          /* Get control of access system functions */
            dssetup(RLStruct.PortNumber);              /* Select port value */

            LRomPtr = dsromdata();        /* Get pointer to ROM data buffer */
            memcpy(LRomPtr, rom_ptr, 8);        /* Copy in callers ROM data */

            if (dsaccess())                /* Select specific device on bus */
            {
               dsdatabyte(0x0F);      /* Send write scratchpad command byte */
               dsdatabyte((uchar) (page_num << 5)); /* Low byte target addr */ 
               dsdatabyte((uchar) (page_num >> 3)); /*High byte target addr */ 

               for (i = 0; i < 32; i++)             /* Read scratchpad data */
                  dsdatabyte(*(scr_dta + i));

               FRes = TRUE;                        /* Set success condition */
            }
            dskeyclose();     /* Release control of access system functions */
         break;

         default:
         break;
      }
   }
   return FRes;                        /* Return result of scratchpad write */
}

uchar read_1427_smem(uchar *smem_dta, uchar *rom_ptr, uchar page_num)
{
   uchar        i;
   uchar        *LRomPtr;
   uchar        FRes;
   RLSType      RLStruct;
   DS1412PACKET SecMemPacket;

   CheckForLock(rom_ptr);   /* If ROM is locked, wait for it to be unlocked */

   FRes = FALSE;                                          /* Assume failure */
   if (FindROMPort(&RLStruct, rom_ptr))            /* See if part was found */
   {
      switch (RLStruct.PortType)             /* What adapter was it in last */
      {
         case SerialAdapter:                   /* Wer'e talking to a DS1412 */
            SecMemPacket.PacketLength = 44;       /* Length of transmission */
            SecMemPacket.RomCommand = 0x55;           /* ROM access command */

            memcpy(SecMemPacket.RomData, rom_ptr, 8);           /* ROM data */

            SecMemPacket.CommandLength = 3;      /* Number of command bytes */

            SecMemPacket.CommandData[0] = 0xF0;  /* Read secure mem command */
            SecMemPacket.CommandData[1] = page_num << 5;      /* Target low */
            SecMemPacket.CommandData[2] = page_num >> 3;     /* Target high */

            SecMemPacket.DataPacketLength = 32;     /* Number of data bytes */

            memset(SecMemPacket.DataPacket, 0xFF, 32);        /* Data bytes */

            FRes = XmitDS1412Data(&SecMemPacket, RLStruct.PortNumber); 
           
            if (FRes)             /* Copy data to user buffer if successful */
               memcpy(smem_dta, SecMemPacket.DataPacket, 32);
         break;

         case ParallelAdapter:     /* Wer'e talking to a DS1410X or DS1410D */
            dskeyopen();          /* Get control of access system functions */
            dssetup(RLStruct.PortNumber);              /* Select port value */

            LRomPtr = dsromdata();        /* Get pointer to ROM data buffer */
            memcpy(LRomPtr, rom_ptr, 8);        /* Copy in callers ROM data */

            if (dsaccess())                /* Select specific device on bus */
            {
               dsdatabyte(0xF0);       /* Send read secure mem command byte */
               dsdatabyte((uchar) (page_num << 5)); /* Low byte target addr */ 
               dsdatabyte((uchar) (page_num >> 3));/* High byte target addr */ 

               for (i = 0; i < 32; i++)                    /* Read RAM data */
                  *(smem_dta + i) = dsdatabyte(0xFF);   

               FRes = TRUE;                        /* Set success condition */
            }
            dskeyclose();     /* Release control of access system functions */
         break;

         default:
         break;
      }
   }
   return FRes;                      /* Return result of secure memory read */
}

uchar copy_1427_sp(uchar *auth_code, uchar *rom_ptr)
{
   uchar        i;
   uchar        *LRomPtr;
   uchar        FRes;
   RLSType      RLStruct;
   DS1412PACKET CpyScrPacket;

   CheckForLock(rom_ptr);   /* If ROM is locked, wait for it to be unlocked */

   FRes = FALSE;                                          /* Assume failure */
   if (FindROMPort(&RLStruct, rom_ptr))            /* See if part was found */
   {
      switch (RLStruct.PortType)             /* What adapter was it in last */
      {
         case SerialAdapter:                   /* Wer'e talking to a DS1412 */
            CpyScrPacket.PacketLength = 13;       /* Length of transmission */
            CpyScrPacket.RomCommand = 0x55;           /* ROM access command */

            memcpy(CpyScrPacket.RomData, rom_ptr, 8);           /* ROM data */

            CpyScrPacket.CommandLength = 1;          /* Command byte number */

            CpyScrPacket.CommandData[0] = 0x55;  /* Copy scratchpad command */
 
            CpyScrPacket.DataPacketLength = 3;      /* Number of data bytes */

            memcpy(CpyScrPacket.DataPacket, auth_code, 3);    /* Auth bytes */

            FRes = XmitDS1412Data(&CpyScrPacket, RLStruct.PortNumber); 
         break;

         case ParallelAdapter:     /* Wer'e talking to a DS1410X or DS1410D */
            dskeyopen();          /* Get control of access system functions */
            dssetup(RLStruct.PortNumber);              /* Select port value */

            LRomPtr = dsromdata();        /* Get pointer to ROM data buffer */
            memcpy(LRomPtr, rom_ptr, 8);        /* Copy in callers ROM data */

            if (dsaccess())                /* Select specific device on bus */
            {
               dsdatabyte(0x55);       /* Send copy scratchpad command byte */

               for (i = 0; i < 3; i++)          /* Send authorization bytes */
                  dsdatabyte(*(auth_code + i));

               FRes = TRUE;                        /* Set success condition */
            }
            dskeyclose();     /* Release control of access system functions */
         break;

         default:
         break;
      }
   }
   return FRes;                         /* Return result of scratchpad copy */
}

uchar FindROMPort(RLSType *WS, uchar *ReqROM)
{
   uchar i = 0;
   uchar FoundIt;

   do                                  /* Try to find ReqROM in global list */
   {
      FoundIt = (memcmp(ROMStr[i]->RomData, ReqROM, 8) == 0);
   }
   while (!FoundIt && (++i < GNum));
   
   if (FoundIt)                        /* Tell caller where we found ReqROM */
   {
      WS->PortType = ROMStr[i]->PortType;            /* Assign adapter type */
      WS->PortNumber = ROMStr[i]->PortNumber;         /* Assign port number */
   }

   return FoundIt;                               /* Return result of search */
}

static uchar SearchPort(uchar PortType, uchar PortNum) 
{
   uchar NumFound;
   uchar i;
   uchar *RomPtr;
   uchar LROMData[8];

   NumFound = 0;                    /* Assume no buttons found on this port */

   if (sm[PortType][PortNum])                 /* Should we search this port */
   {
      switch (PortType)
      {
         case SerialAdapter:                        /* Try to find a DS1412 */
            if (SerialRS(LROMData, PortNum, FIRSTBUTTON))/*Find first button*/ 
            {
               do
               {
                  for (i = 0; i < 8; i++)    /* Copy bytes to global buffer */
                     *(GRomPtr + i) = LROMData[i]; 

                  GRomPtr += 8;      /* Point to next empty space in buffer */
                  ++NumFound;
               }
               while (SerialRS(LROMData, PortNum, NEXTBUTTON));/*Next button*/ 
            }
         break;
   
         case ParallelAdapter:

            dssetup(PortNum);          /* Set port address in access system */
            dskeyopen();                            /* Gain control of port */
            
            if (dsgndtest())                   /* See if we can communicate */
            {
               if (dsfirst())         /* Look for first button on this port */
               {
                  do
                  {
                     RomPtr = dsromdata();/*Get pointer to this buttons ROM */
                     
                     for (i = 0; i < 8; i++) /* Copy bytes to global buffer */
                        *(GRomPtr + i) = *(RomPtr + i);
   
                     GRomPtr += 8;   /* Point to next empty space in buffer */
                     ++NumFound;                /* Found yet another button */
                  }
                  while (dsnext());      /* Look for next part on this port */
               }
            }
            dskeyclose();                        /* Release control of port */
         break;
   
         default:
         break;
      }
   }
   return NumFound;          /* Return number of buttons found on this port */
}

void ReadConfig(void)                       /* Read port configuration file */
{
   uchar i, j, sc;
   uchar AdapterNum, FPortNum;
   FILE  *FDes;
   int   TPNum; 
   char  SName[40];

   for (i = 0; i < MAX_PARALLEL_PORTS; i++)    /* Search all parallel ports */
      sm[ParallelAdapter][i] = TRUE;

   for (i = 0;i < MAX_SERIAL_PORTS;i++) /*Search only specified serial ports*/
      sm[SerialAdapter][i] = FALSE;

   FDes = fopen("/portcfg/pcf.cfg", "rb");            /* See if file exists */

   if (FDes)                  /* If file existed, see which ports to search */
   {
      while (fread(&AdapterNum, 1, 1, FDes))           /* Read adapter type */
      {
         if (AdapterNum < (uchar) LastAdapter)      /* Valid adapter type ? */
         {
            fread(&sc, 1, 1, FDes);                       /* Read seperator */
            fread(&FPortNum, 1, 1, FDes);           /* Read number of ports */ 
            fread(&sc, 1, 1, FDes);                       /* Read seperator */

            if (FPortNum < MaxPortNum[AdapterNum])
            {
               for (i = 0; i < FPortNum; i++)     /* Loop through all ports */
               {
                  j = 0;
                  do                      /* Build string representing port */
                  {
                     fread(&sc, 1, 1, FDes);
                     SName[j++] = sc;
                  }
                  while (sc != 0x7E);                 /* Look for seperator */
                  SName[j - 1] = 0;                     /* Terminate string */

                  SPName[i] = malloc(strlen(SName));         /* Grab memory */
                  strcpy(SPName[i], SName);          /* Copy in device name */

                  sm[AdapterNum][i] = TRUE;     /* Want to search this port */
                  ++SPNum;    /* Increment number of serial ports to search */
               }    
            }
         }  
      }
   }
}



