/*
         satunxx.c                                        DS1410X stuff

               COPYRIGHT (C) 1994 DALLAS SEMICONDUCTOR COPORATION
*/

#include <fcntl.h>  
#include <signal.h> 
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/sem.h>

#ifndef TRUE

#define TRUE 1
#define FALSE 0

#endif 

#define DS1410_KEY 0x130506C4;                /* Just some number for a key */
#define OUR_SEM_PERMS 0x1b6; /*We want any process to be allowed sem access */

typedef unsigned char uchar;

static uchar setup_ok = FALSE;                        /* Access system vars */
static uchar fail_next = FALSE;
static uchar access_good = FALSE;
static uchar bits[65], l0 = 0;
static uchar romdta[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };
static int filedes;
static uchar gpn = 1;
static uchar authflag = FALSE;
static uchar mch[3];

static uchar sem_init = FALSE;                            /* Semaphore vars */
static key_t DS1410_key;                                        
static int sem_id;          
static struct sembuf sops;

static runawaysem()                       /* process went wild with keyopen */
{
   semctl(sem_id, 0, SETVAL, 1);                    /* Set semval back to 1 */
}

uchar dskeyopen()
{
   int semflag;
   key_t DS1410_key;

   if (!sem_init)
   {
      DS1410_key = DS1410_KEY;
      semflag = IPC_CREAT | IPC_EXCL | OUR_SEM_PERMS; 
      sem_id = semget(DS1410_key, 1, semflag); 

      if (sem_id == -1)                                  /* Already created */
      { 
         semflag = IPC_EXCL | OUR_SEM_PERMS; 
         sem_id = semget(DS1410_key, 1, semflag); 
      }
      else
         semctl(sem_id, 0, SETVAL, 1);                 /* Initialize semval */

      sem_init = TRUE;
   }

   if (!authflag)
   {
      sops.sem_num = 0;
      sops.sem_op = -1;
      sops.sem_flg = SEM_UNDO;

      signal(SIGALRM, runawaysem);  /* If we don't get key in 9 sec. grab it */
      alarm(9);

      semop(sem_id, &sops, 1);

      alarm(0);
      signal(SIGALRM, SIG_IGN);

      filedes = open("/dev/sauthx0", O_RDWR, 1);
      authflag = TRUE;
   } 
   return authflag;
}

uchar dskeyclose()
{
   if (authflag)
   {
      semctl(sem_id, 0, SETVAL, 1);                 /* Set semval back to 1 */

      close(filedes);

      authflag = FALSE;

      nap(50);
   } 
   return TRUE;
}

static uchar doreset()
{
   if (authflag)
   {
      mch[0] = 1 + gpn; 

      write(filedes, mch, 2);

      return mch[0];
   } 
   return 0;
}

static uchar xmitbit(bit_val)            /* Send bit value to device driver */

uchar bit_val;

{
   if (authflag)
   {
      mch[0] = bit_val; 

      write(filedes, mch, 2);

      return mch[0];
   } 
   return 0;
}

static uchar xmitbyte(byte_val)

uchar byte_val;

{
   if (authflag)
   {
      mch[0] = 5;                       /* Tell driver to read/write a byte */
      mch[1] = byte_val;                              /* Specify byte value */

      write(filedes, mch, 2); 

      return mch[0];
   } 
   return byte_val;
}

uchar dssetup(pn)                             /* Set global LPT port number */ 

uchar pn;

{
   setup_ok = FALSE;

   ++pn;
   if (pn > 0 && pn < 4)
   {
      setup_ok = TRUE;
      gpn = pn;
   }
   return setup_ok;
}

/*
      This function performs a ROM search and finds one part on the DOW bus
   per call.
*/

static uchar rs() 
{
   uchar xi = 0, x = 0, ld = l0, tr;

   if (tr = doreset() && (xmitbyte(0xF0))) 
   {
      while (xi++ < 64 && (x = ((xmitbit(1) << 1) + xmitbit(1))) < 3) 
      {
         if (!x) 
         {
            if (xi >= ld) 
               bits[xi] = (xi == ld);

            if (!bits[xi]) 
               l0 = xi;
         } 
         else 
            bits[xi] = (x & 1) ^ 1;

         xmitbit(bits[xi]);
      }
   }

   if (x == 3)
      return 0;

   return !tr ? 0 : 1 + (ld == l0);   
}

static shiftem()
{
   uchar j, k;

   for (j = 0; j < 8; j++)
   {
      romdta[j] = 0;

      for (k = 1; k < 9; k++)
         romdta[j] = romdta[j] + (bits[(j << 3) + k] << (k - 1));
   }
}

uchar dsnext()             /* Find next DOW part on specified parallel port */
{
   uchar tr, xi;

   if (setup_ok)
   {
      if (fail_next)
      {
         fail_next = FALSE; 
         l0 = 0;                                     /* Reset next function */
      } 
      else while ((tr = rs()) != 0)             /* Do that ROM search thang */
      {
         shiftem();             /* Get bit array into 8 byte ROM data array */

         if (tr == 2) 
            fail_next = 1;                /* See if we should force failure */
         
         if ((romdta[0] & 0x80) && !(romdta[5] & 0xF0) && !romdta[6])
            return TRUE;                                       /* Good part */
          
         for (xi = 0; xi < 8; xi++) romdta[xi] = 0; /* Clear ROM data array */
      }
   }
   return 0;                                                     /* No luck */
}

uchar dsfirst()           /* Find first DOW part on specified parallel port */
{
   fail_next = FALSE;                         /* Don't force a failure here */
   l0 = 0;                                    /* Point algorithm to the top */

   return dsnext();            /* Go look for the first DOW part on the bus */
}

uchar dsaccess()
{
   uchar xi, yi;

   access_good = FALSE;

   if (doreset())                                       /* Send reset pulse */
   {
      xmitbyte(0xF0);                            /* ROM search command byte */
      for (xi = 0; xi < 8; xi++) 
      {
         for (yi = 0; yi < 8; yi++)
         {
            if (((xmitbit(1) << 1) | xmitbit(1)) == 3)                
               return FALSE;                          /* Part not there man */

            xmitbit((romdta[xi] >> yi) & 1);        /* Send write time slot */ 
         }      
      }
      access_good = TRUE;
   }
   return access_good;
}

uchar dsdatabyte(br)              /* Read/Write data to buttons on the port */

uchar br;

{
   return (setup_ok && access_good) ? xmitbyte(br) : br;
}

uchar dsgndtest()
{
   if (setup_ok)
   {
      doreset();
      return xmitbit(1); 
   }
   return FALSE;
}

uchar *dsromdata()
{
   return romdta;                      /* Return pointer to ROM Data buffer */ 
}


uchar dowcheck()                 /* See if device driver has been installed */
{
   if (!authflag)
      return FALSE;

   filedes = open("/dev/sauth0", O_RDWR, 1);
   close(filedes);

   return (filedes == -1) ? FALSE : TRUE;
}      




