/*
          Serial port transfer layer               Ver 0.01 2/24/94

              COPYRIGHT (C) 1994 DALLAS SEMICONDUCTOR COPORATION
*/

#include <fcntl.h>
#include <sys/termio.h>

#include "dbapi.h"

#define FIRST    0xC3                               /* DS1412 first command */
#define SLOWNEXT 0xC5                                /* DS1412 next command */
#define RETRYNUM 4            /* Number of PORs for DS1412 before giving up */

#define START_KEY 0x1B030F8F                  /* just some number for a key */

uchar GetSPAccess(void);                    /* Internal function prototypes */
uchar ReleaseSPAccess(void); 
uchar WriteCommBytes(uchar *SendBuff, uchar ByteNum);
uchar ReadCommBytes(uchar *RecvBuff, uchar ByteNum);
void  ResetDS1412(void);
void  InitStuff(void);

union semss 
{
   int val;
   struct semid_ds *buf;
   ushort *array;
}; 

key_t         DS1412_key; /* Semaphore contol/operation variables and flags */
int           sem_id;                                       /* Semaphore id */
int           fdes;                                           /* TTY handle */
struct termio *argnew;        /* New serial port settings in this structure */
struct sembuf sops;                       /* Semaphore operations structure */
uchar         TimeOut;                                     /* Time out flag */
uchar         rsp;                  /* ROM search pointer to send to DS1412 */
uchar         FailNext = FALSE;                    /* used to force failure */
uchar         DS1412Resets;      /* Keeps track of reset attempts on DS1412 */
extern char   *SPName[MAX_SERIAL_PORTS];                /* TTY device names */
extern uchar  SPNum;                           /* Number of ports to search */
uchar         cpn;                                           /* Port number */

uchar SerialRS(uchar *RomFound, uchar PortNum, uchar FoNFlag)
{
   uchar llength = 0;
   uchar CommandByte;

   DS1412Resets = 0;                              /* Initialize reset count */

   if (FailNext)                      /* Last button was found on last call */
   {
      FailNext = FALSE;                                      /* Reset state */
      return FALSE;
   }

   cpn = PortNum;
   if (GetSPAccess())                        /* Gain control of serial port */
   {
      do
      {
         TimeOut = FALSE;                       /* Initialize time out flag */
         
         CommandByte = (FoNFlag == FIRSTBUTTON) ? FIRST : SLOWNEXT;

         WriteCommBytes(&CommandByte, 1);              /* Send command byte */ 
      
         if (FoNFlag == NEXTBUTTON)                    /* Send old ROM data */
         {
            WriteCommBytes(&rsp, 1);             /* Send ROM search pointer */
            WriteCommBytes(RomFound, 8);                   /* Send ROM data */
         }
         
         ReadCommBytes(&llength, 1); /* Read length of receive transmission */
   
         if ((llength == 9) && !TimeOut)
         {
            ReadCommBytes(&rsp, 1);              /* Read ROM search pointer */
            ReadCommBytes(RomFound, 8);                    /* Read ROM data */
      
            if (rsp == 0x41)           /* See if last button has been found */
               FailNext = TRUE; /* We want to return FALSE next time called */
         }
      }
      while (TimeOut && (DS1412Resets++ < RETRYNUM));
   }

   ReleaseSPAccess();                    /* Release access to this com port */

   return ((llength > 0) && !TimeOut);           /* Success if button found */
}

uchar XmitDS1412Data(PDS1412PACKET PDS1412Packet, uchar PortNum)
{
   uchar i, j = 0, llength = 0, pl;
   uchar DataOffset;
   uchar *TranCopy, *DataPacket;

   DS1412Resets = 0;                              /* Initialize reset count */
   DataPacket = (uchar *) malloc(sizeof(DS1412PACKET)); 

   *(DataPacket + j++) = pl = PDS1412Packet->PacketLength;/*Set trans length*/
   *(DataPacket + j++) = PDS1412Packet->RomCommand;     /* ROM command byte */

   for (i = 0; i < 8; i++)                              /* Copy in ROM data */
      *(DataPacket + j++) = PDS1412Packet->RomData[i];

   for (i = 0; i < PDS1412Packet->CommandLength; i++)  /* Copy command data */
      *(DataPacket + j++) = PDS1412Packet->CommandData[i];

   for (i = 0; i < PDS1412Packet->DataPacketLength; i++) /* Copy Data bytes */
      *(DataPacket + j++) = PDS1412Packet->DataPacket[i];
   
   cpn = PortNum;
   if (GetSPAccess())                     /* Aquire access to this com port */
   {
      TranCopy = (uchar *) malloc(*DataPacket + 1); /* Grab memory for copy */
   
      do
      {
         memcpy(TranCopy, DataPacket, (uchar) (*DataPacket + 1)); 

         TimeOut = FALSE;                       /* Initialize time out flag */
      
         WriteCommBytes(TranCopy, 1);               /* Send the length byte */
         ReadCommBytes(&llength, 1);                   /* Read return value */ 

         if ((llength == pl) && !TimeOut)             /* Send/Receive bytes */
         {
            WriteCommBytes(TranCopy + 1, llength);       /* Write the stuff */
            ReadCommBytes(TranCopy + 1, llength);         /* Read the stuff */
         }
      }
      while (TimeOut && (DS1412Resets++ < RETRYNUM));

      if (!TimeOut && llength)     /* Copy results into user supplied array */
      {
         DataOffset = 10 + PDS1412Packet->CommandLength; /*Data packet offs */

         memcpy(PDS1412Packet->DataPacket,         /* Only copy data packet */
                TranCopy + DataOffset, 
                PDS1412Packet->DataPacketLength);
      }

      free(TranCopy);                     /* Free memory used by copy array */
   }

   ReleaseSPAccess();                    /* Release access to this com port */
   
   return ((llength > 0) && !TimeOut);                     /* Return status */
}

semlocked()                   /* Called in the event of a runaway semaphore */
{
   semctl(sem_id, cpn, SETVAL, 1);                  /* Set semval back to 1 */
}

void InitStuff(void)
{
   int         semflg;
   uchar       i;
   ushort      InitSemVals[MAX_SERIAL_PORTS];
   union semss InitVal;

   argnew = (struct termio *) malloc(sizeof(struct termio));

   DS1412_key = START_KEY;
   semflg = IPC_CREAT | IPC_EXCL | OUR_SEM_PERMS;
   sem_id = semget(DS1412_key, SPNum, semflg);

   if (sem_id == -1)                                   /* May already exist */
   {
      semflg = IPC_EXCL | OUR_SEM_PERMS;
      sem_id = semget(DS1412_key, SPNum, semflg);
   }
   else                      /* If just created initialize semaphore values */
   {
      for (i = 0; i < SPNum; i++)         /* Set semvals for all ports to 1 */
         InitSemVals[i] = 1;

      InitVal.array = InitSemVals; 

      semctl(sem_id, 0, SETALL, InitVal);             /* Initialize semvals */
   }
}

uchar GetSPAccess()
{
   static uchar FirstTime = TRUE;

   if (FirstTime)
   {
      InitStuff();
      FirstTime = FALSE;
   }

   sops.sem_num = cpn;
   sops.sem_op = -1;
   sops.sem_flg = SEM_UNDO;

   signal(SIGALRM, semlocked);
   alarm(10);

   semop(sem_id, &sops, 1); 

   alarm(0);
   signal(SIGALRM, SIG_IGN); 

   if ((fdes = open(SPName[cpn], O_RDWR)) == -1)    /* Handle to tty driver */
      return FALSE;

   if (ioctl(fdes, TCGETA, argnew) == -1)             /* Get tty paramaters */
      return FALSE;

   argnew->c_iflag = 0;                           /* Set new tty paramaters */
   argnew->c_cflag = B9600 | CS8 | CLOCAL | CREAD;
   argnew->c_lflag = 0;
   argnew->c_oflag = 0;
   argnew->c_cc[0] = 0; 
   argnew->c_cc[1] = 0; 
   argnew->c_cc[2] = 0; 
   argnew->c_cc[3] = 0; 
   argnew->c_cc[4] = 0;                             /* No character minimum */
   argnew->c_cc[5] = 2;                               /* .2 second time out */
   argnew->c_cc[7] = 0;  

   return (ioctl(fdes, TCSETAF, argnew) == -1) ? FALSE : TRUE;
}

uchar ReleaseSPAccess()
{
   semctl(sem_id, cpn, SETVAL, 1);                  /* Set semval back to 1 */
   close(fdes);                              /* were done with port for now */ 
   
   return TRUE;       
}

uchar ReadCommBytes(uchar *RecvBuff, uchar ByteNum)
{
   uchar i = 0;

   do
   {
      TimeOut = (read(fdes, RecvBuff + i, 1) != 1); /* Read from tty driver */
   }
   while (!TimeOut && (++i < ByteNum));

   if (TimeOut)                    /* Try to reset device if error occurred */
      ResetDS1412();

   return !TimeOut;              /* Did we get all the bytes we asked for ? */ 
} 

uchar WriteCommBytes(uchar *SendBuff, uchar ByteNum)
{
   TimeOut = (write(fdes, SendBuff, ByteNum) != ByteNum);    /* Send to tty */

   if (TimeOut)                    /* Try to reset device if error occurred */
      ResetDS1412();

   return !TimeOut;              /* Did we get all the bytes we asked for ? */ 
}

void ResetDS1412(void)
{
   ushort LowPulseWidth, HighPulseWidth;

   LowPulseWidth = (DS1412Resets < 3) ? 40 : 750;     /* Determine low time */
   HighPulseWidth = (DS1412Resets < 3) ? 20 : 150;   /* Determine high time */

   ioctl(fdes, TCGETA, argnew);                   /* get serial port params */
   argnew->c_cflag = B0 | CS8 | CREAD;        /* 0 baud rate drops RTS, DTR */
   ioctl(fdes, TCSETAF, argnew);                           /* Drop RTS, DTR */

   close(fdes);   
   nap(LowPulseWidth);                                 /* Wait with Vcc low */
   fdes = open(SPName[cpn], O_RDWR);

   ioctl(fdes, TCGETA, argnew);                   /* get serial port params */
   argnew->c_cflag = B9600 | CS8 | CREAD | CLOCAL;       /* Reset baud rate */
   argnew->c_cc[4] = 0;                              /* 0 = min char number */
   argnew->c_cc[5] = 2;                               /* .2 secone time out */
   ioctl(fdes, TCSETAF, argnew);                            /* Set RTS, DTR */

   nap(HighPulseWidth);                               /* Wait with Vcc high */
}




